@@ -1,4 +1,4 @@
-/*                             ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+/*
   EightyOne - A simple Sudoku solving game
   Copyright (C) 2006  Tim Teulings
 
@@ -32,7 +32,7 @@
 #include <Lum/Popup.h>
 #include <Lum/Text.h>
 #include <Lum/Toggle.h>
-
+#include <iostream>
 static size_t popupFontSize = 200;
 
 class SetSelector : public Lum::Popup
@@ -186,7 +186,6 @@ public:
 
 GameArea::GameArea()
  : minFont(Lum::OS::display->GetFont()),
-   minMarkFont(Lum::OS::display->GetFontByPixel(Lum::OS::Display::fontTypeProportional,7)),
    font(Lum::OS::display->GetFont(Lum::OS::Display::fontScaleCaption1)),
    presetColor(0.5,0.5,0.5,Lum::OS::display->GetColor(Lum::OS::Display::whiteColor)),
    focusBackgroundColor(0.8,0.8,0.8,Lum::OS::display->GetColor(Lum::OS::Display::fillColor)),
@@ -194,16 +193,19 @@ GameArea::GameArea()
    correctColor(0,0.5,0,Lum::OS::display->GetColor(Lum::OS::Display::blackColor)),
    hintColor(1,1,0,Lum::OS::display->GetColor(Lum::OS::Display::blackColor)),
    boxSize(0),
-   focusedCell(0),
-   focusChangedAction(new Lum::Model::Action()),
-   hintMode(new Lum::Model::Boolean()),
-   finished(new Lum::Model::Boolean())
+   hintMode(new Lum::Model::Boolean())
 {
+  if (Lum::OS::display->GetSize()<Lum::OS::Display::sizeNormal) {
+    minMarkFont=Lum::OS::display->GetFontByPixel(Lum::OS::Display::fontTypeProportional,7);
+  }
+  else {
+    minMarkFont=Lum::OS::display->GetFont(Lum::OS::Display::fontScaleFootnote);
+  }
+
   SetCanFocus(true);
   RequestFocus();
 
   hintMode->Set(false);
-  finished->Set(true);
 
   Observe(hintMode);
 }
@@ -213,95 +215,13 @@ int GameArea::GetCellOffset(size_t index) const
   return (index+1)*1+(index/3+1)*1+index*boxSize;
 }
 
-void GameArea::SetFinished(bool finished)
+bool GameArea::SetModel(Lum::Base::Model* model)
 {
-  this->finished->Set(finished);
+  this->model=dynamic_cast<GameModel*>(model);
 
-    hintMode->Set(false);
-    if (finished) {
-      hintMode->Disable();
-    }
-    else {
-      hintMode->Enable();
-    }
-}
+  Control::SetModel(this->model);
 
-void GameArea::SetModel(const Sudoku &riddle, const Sudoku &solution)
-{
-  this->solution=solution;
-  this->riddle=riddle;
-  this->game=riddle;
-
-  for (size_t i=0; i<Sudoku::size; i++) {
-    marks[i]=0;
-  }
-
-  SetFinished(game==solution);
-  focusedCell=0;
-
-  Redraw();
-  focusChangedAction->Trigger();
-}
-
-void GameArea::SetGame(const Game& game)
-{
-  riddle.SetAreaAsString(game.riddle);
-  solution.SetAreaAsString(game.solution);
-  this->game.SetAreaAsString(game.game);
-
-  for (size_t i=0; i<Sudoku::size; i++) {
-    marks[i]=0;
-  }
-
-  if (game.marks.length()==9*Sudoku::size) {
-    for (size_t i=0; i<9*Sudoku::size; i++) {
-      if (game.marks[i]!=L' ') {
-        marks[i/9]|=(1 << (i%9));
-      }
-    }
-  }
-
-  SetFinished(this->game==solution);
-
-  focusedCell=0;
-  Redraw();
-
-  focusChangedAction->Trigger();
-}
-
-void GameArea::GetGame(Game& game) const
-{
-  riddle.GetAreaAsString(game.riddle);
-  solution.GetAreaAsString(game.solution);
-  this->game.GetAreaAsString(game.game);
-
-  game.marks=L"";
-
-  for (size_t i=0; i<Sudoku::size; i++) {
-    for (size_t j=0; j<=8; j++) {
-      if (marks[i] & (1 << j)) {
-        game.marks.append(L"*");
-      }
-      else {
-        game.marks.append(L" ");
-      }
-    }
-  }
-}
-
-void GameArea::GetGame(Sudoku& game) const
-{
-  game=this->game;
-}
-
-void GameArea::GetRiddle(Sudoku& game) const
-{
-  game=this->riddle;
-}
-
-void GameArea::GetSolution(Sudoku& game) const
-{
-  game=this->solution;
+  return this->model.Valid();
 }
 
 void GameArea::CalcSize()
@@ -368,7 +288,7 @@ Lum::OS::Font* GameArea::FindFont(Lum::OS::Font* minFont, size_t size)
       for (size_t i=1; i<=Sudoku::dimension; i++) {
         boxSize=std::max(boxSize,tmpFont->StringWidth(Lum::Base::NumberToWString(i),
                                                       Lum::OS::Font::bold));
-        boxSize=std::max(boxSize,tmpFont->pixelHeight);
+        boxSize=std::max(boxSize,tmpFont->height);
       }
 
       fits=boxSize<=size && fontSize<=size;
@@ -442,6 +362,10 @@ void GameArea::Draw(Lum::OS::DrawInfo* draw,
   }
   draw->PopForeground();
 
+  if (!model.Valid()) {
+    return;
+  }
+
   //
   // Draw values
   //
@@ -449,8 +373,9 @@ void GameArea::Draw(Lum::OS::DrawInfo* draw,
   for (size_t i=0; i<Sudoku::dimension*Sudoku::dimension; i++) {
 
     // Draw background
-    if ((HasFocus() || true/*Lum::OS::display->GetTheme()->RequestFingerFriendlyControls()*/)
-        && !finished->Get() && i==focusedCell) {
+    if ((HasFocus() || true/*Lum::OS::display->GetTheme()->RequestFingerFriendlyControls()*/) &&
+        !model->IsFinished() &&
+        i==model->GetFocusedCell()) {
       draw->PushForeground(focusBackgroundColor);
     }
     else {
@@ -461,25 +386,26 @@ void GameArea::Draw(Lum::OS::DrawInfo* draw,
                         boxSize,boxSize);
     draw->PopForeground();
 
-    if (riddle.Get(i)!=Sudoku::empty) { // Preset fields
-      std::wstring        text(Lum::Base::NumberToWString(riddle.Get(i)));
+    if (model->GetRiddle(i)!=Sudoku::empty) { // Preset fields
+      std::wstring        text(Lum::Base::NumberToWString(model->Get(i)));
       Lum::OS::FontExtent extent;
 
       font->StringExtent(text,extent,Lum::OS::Font::bold);
 
       draw->PushForeground(presetColor);
       draw->DrawString(this->x+GetCellOffset(i%Sudoku::dimension)+(boxSize-extent.width+extent.left+extent.right)/2-extent.left,
-                       this->y+GetCellOffset(i/Sudoku::dimension)+(boxSize-extent.height)/2+extent.ascent,
+                       this->y+GetCellOffset(i/Sudoku::dimension)+(boxSize-font->height)/2+font->ascent,
                        text);
       draw->PopForeground();
     }
-    else if (finished->Get() && solution.Get(i)!=Sudoku::empty) {
-      std::wstring        text(Lum::Base::NumberToWString(solution.Get(i)));
+    else if (model->IsValid() &&
+             model->IsFinished()) {
+      std::wstring        text(Lum::Base::NumberToWString(model->GetSolution(i)));
       Lum::OS::FontExtent extent;
 
       font->StringExtent(text,extent,Lum::OS::Font::bold);
 
-      if (solution.Get(i)!=game.Get(i)) {
+      if (model->GetSolution(i)!=model->Get(i)) {
         draw->PushForeground(errorColor);
       }
       else {
@@ -487,20 +413,20 @@ void GameArea::Draw(Lum::OS::DrawInfo* draw,
       }
 
       draw->DrawString(this->x+GetCellOffset(i%Sudoku::dimension)+(boxSize-extent.width+extent.left+extent.right)/2-extent.left,
-                       this->y+GetCellOffset(i/Sudoku::dimension)+(boxSize-extent.height)/2+extent.ascent,
+                       this->y+GetCellOffset(i/Sudoku::dimension)+(boxSize-font->height)/2+font->ascent,
                        text);
       draw->PopForeground();
     }
-    else if (game.Get(i)!=Sudoku::empty) { // manually set fields
-      std::wstring         text(Lum::Base::NumberToWString(game.Get(i)));
+    else if (model->Get(i)!=Sudoku::empty) { // manually set fields
+      std::wstring         text(Lum::Base::NumberToWString(model->Get(i)));
       Lum::OS::FontExtent  extent;
       Sudoku::Bitset       allowed;
 
-      game.GetPotential(i,allowed);
+      model->GetPotential(i,allowed);
 
       font->StringExtent(text,extent,Lum::OS::Font::bold);
 
-      if (!allowed.test(game.Get(i))) {
+      if (!allowed.test(model->Get(i))) {
         draw->PushForeground(errorColor);
       }
       else {
@@ -508,27 +434,26 @@ void GameArea::Draw(Lum::OS::DrawInfo* draw,
       }
 
       draw->DrawString(this->x+GetCellOffset(i%Sudoku::dimension)+(boxSize-extent.width+extent.left+extent.right)/2-extent.left,
-                       this->y+GetCellOffset(i/Sudoku::dimension)+(boxSize-extent.height)/2+extent.ascent,
+                       this->y+GetCellOffset(i/Sudoku::dimension)+(boxSize-font->height)/2+font->ascent,
                        text);
       draw->PopForeground();
     }
-    else if (!finished->Get() &&
-             hintMode->Get() &&
-             i==focusedCell) { // Empty fields with solution hint requested
-      std::wstring        text(Lum::Base::NumberToWString(solution.Get(i)));
+    else if (hintMode->Get() &&
+             i==model->GetFocusedCell()) { // Empty fields with solution hint requested
+      std::wstring        text(Lum::Base::NumberToWString(model->GetSolution(i)));
       Lum::OS::FontExtent extent;
 
       font->StringExtent(text,extent,Lum::OS::Font::bold);
 
       draw->PushForeground(hintColor);
       draw->DrawString(this->x+GetCellOffset(i%Sudoku::dimension)+(boxSize-extent.width+extent.left+extent.right)/2-extent.left,
-                       this->y+GetCellOffset(i/Sudoku::dimension)+(boxSize-extent.height)/2+extent.ascent,
+                       this->y+GetCellOffset(i/Sudoku::dimension)+(boxSize-font->height)/2+font->ascent,
                        text);
       draw->PopForeground();
     }
     else { // Marked fields
       for (size_t j=0; j<Sudoku::dimension; j++) {
-        if (marks[i] & (1 << j)) {
+        if (model->GetMarks(i) & (1 << j)) {
           std::wstring         text(Lum::Base::NumberToWString(j+1));
           Lum::OS::FontExtent  extent;
 
@@ -541,7 +466,7 @@ void GameArea::Draw(Lum::OS::DrawInfo* draw,
                            (boxSize/3-extent.width+extent.left+extent.right)/2-extent.left,
                            this->y+GetCellOffset(i/Sudoku::dimension)+
                            (boxSize/3)*(j/3)+
-                           (boxSize/3-extent.height)/2+extent.ascent,
+                           (boxSize/3-markFont->height)/2+markFont->ascent,
                            text);
           draw->PopForeground();
           draw->PopFont();
@@ -556,23 +481,22 @@ bool GameArea::HandlesClickFocus() const
 {
   return CanFocus() &&
          IsVisible() &&
-         finished.Valid() &&
-         !finished->IsNull() &&
-         !finished->Get();
+         model.Valid() &&
+         !model->IsFinished();
 }
 
 bool GameArea::HandlesKeyFocus() const
 {
   return CanFocus() &&
          IsVisible() &&
-         finished.Valid() &&
-         !finished->IsNull() &&
-         !finished->Get();
+         model.Valid() &&
+         !model->IsFinished();
 }
 
 bool GameArea::HandleMouseEvent(const Lum::OS::MouseEvent& event)
 {
-  if (finished->Get()) {
+  if (!model.Valid() ||
+      model->IsFinished()) {
     return false;
   }
 
@@ -591,14 +515,12 @@ bool GameArea::HandleMouseEvent(const Lum::OS::MouseEvent& event)
       tmp++;
     }
 
-    if (tmp<Sudoku::dimension*Sudoku::dimension) {
-      if (tmp!=focusedCell) {
-        focusedCell=tmp;
-        Redraw();
-        focusChangedAction->Trigger();
+    if (tmp<Sudoku::size) {
+      if (tmp!=model->GetFocusedCell()) {
+        model->SetFocusedCell(tmp);
       }
 
-      if (riddle.Get(focusedCell)!=Sudoku::empty) {
+      if (model->Get(model->GetFocusedCell())!=Sudoku::empty) {
         return true;
       }
 
@@ -612,7 +534,7 @@ bool GameArea::HandleMouseEvent(const Lum::OS::MouseEvent& event)
           event.y-(this->y+GetCellOffset(tmp/Sudoku::dimension))<=(int)boxSize/2) {
         MarkSelector *selector;
 
-        selector=new MarkSelector(marks[focusedCell]);
+        selector=new MarkSelector(model->GetFocusedCellMarks());
         selector->SetParent(GetWindow());
         selector->SetPosition(GetWindow()->GetX()+event.x+5,GetWindow()->GetY()+event.y+5);
         selector->GetWindow()->SetOpacity(0.8);
@@ -621,12 +543,7 @@ bool GameArea::HandleMouseEvent(const Lum::OS::MouseEvent& event)
           selector->EventLoop();
           selector->Close();
 
-          marks[focusedCell]=selector->GetValue();
-          if (marks[focusedCell]!=0) {
-            game.Set(focusedCell,Sudoku::empty);
-          }
-
-          Redraw();
+          model->SetFocusedCellMarks(selector->GetValue());
         }
 
         delete selector;
@@ -643,10 +560,7 @@ bool GameArea::HandleMouseEvent(const Lum::OS::MouseEvent& event)
         selector->Close();
 
         if (selector->HasSelection()) {
-          game.Set(focusedCell,selector->GetValue());
-          marks[focusedCell]=0;
-          SetFinished(game==solution);
-          Redraw();
+          model->SetFocusedCellValue(selector->GetValue());
         }
 
         delete selector;
@@ -663,7 +577,8 @@ bool GameArea::HandleMouseEvent(const Lum::OS::MouseEvent& event)
 
 bool GameArea::HandleKeyEvent(const Lum::OS::KeyEvent& event)
 {
-  if (finished->Get()) {
+  if (!model.Valid() ||
+      model->IsFinished()) {
     return false;
   }
 
@@ -672,52 +587,38 @@ bool GameArea::HandleKeyEvent(const Lum::OS::KeyEvent& event)
   if (event.type==Lum::OS::KeyEvent::down) {
     switch (event.key) {
     case Lum::OS::keyLeft:
-      if (focusedCell%Sudoku::dimension>0) {
-        focusedCell--;
-        Redraw();
-        focusChangedAction->Trigger();
+      if (model->GetFocusedCell()%Sudoku::dimension>0) {
+        model->SetFocusedCell(model->GetFocusedCell()-1);
       }
       return true;
     case Lum::OS::keyRight:
-      if (focusedCell%Sudoku::dimension<Sudoku::dimension-1) {
-        focusedCell++;
-        Redraw();
-        focusChangedAction->Trigger();
+      if (model->GetFocusedCell()%Sudoku::dimension<Sudoku::dimension-1) {
+        model->SetFocusedCell(model->GetFocusedCell()+1);
       }
       return true;
     case Lum::OS::keyUp:
-      if (focusedCell/Sudoku::dimension>0) {
-        focusedCell-=Sudoku::dimension;
-        Redraw();
-        focusChangedAction->Trigger();
+      if (model->GetFocusedCell()/Sudoku::dimension>0) {
+        model->SetFocusedCell(model->GetFocusedCell()-Sudoku::dimension);
       }
       return true;
     case Lum::OS::keyDown:
-      if (focusedCell/Sudoku::dimension<Sudoku::dimension-1) {
-        focusedCell+=Sudoku::dimension;
-        Redraw();
-        focusChangedAction->Trigger();
+      if (model->GetFocusedCell()/Sudoku::dimension<Sudoku::dimension-1) {
+        model->SetFocusedCell(model->GetFocusedCell()+Sudoku::dimension);
       }
       return true;
     case Lum::OS::keySpace:
     case Lum::OS::keyBackspace:
     case Lum::OS::keyDelete:
-      if (riddle.Get(focusedCell)==Sudoku::empty) {
-        game.Set(focusedCell,Sudoku::empty);
-        Redraw();
-        focusChangedAction->Trigger();
+      if (model->IsFocusedCellEditable()) {
+        model->SetFocusedCellValue(Sudoku::empty);
       }
       return true;
     default:
       if (Lum::Base::WStringToNumber(event.text,value)) {
-        if (value>=0 && value<=Sudoku::dimension) {
-          if (riddle.Get(focusedCell)==Sudoku::empty) {
-            game.Set(focusedCell,value);
-            SetFinished(game==solution);
-            Redraw();
-            // Focus did not changed, but we simulate it to get the value change
-            // notified
-            focusChangedAction->Trigger();
+        if (value>=0 &&
+            value<=Sudoku::dimension) {
+          if (model->IsFocusedCellEditable()) {
+            model->SetFocusedCellValue(value);
           }
         }
 
@@ -730,101 +631,19 @@ bool GameArea::HandleKeyEvent(const Lum::OS::KeyEvent& event)
   return false;
 }
 
-Lum::Model::Action* GameArea::GetFocusChangedAction() const
-{
-  return focusChangedAction.Get();
-}
-
 Lum::Model::Boolean* GameArea::GetHintMode() const
 {
   return hintMode.Get();
 }
 
-Lum::Model::Boolean* GameArea::GetFinishedModel() const
-{
-  return finished.Get();
-}
-
-bool GameArea::IsFocusedCellEditable() const
-{
-  return !finished->Get() && riddle.Get(focusedCell)==Sudoku::empty;
-}
-
-bool GameArea::IsCellInMarkModus() const
-{
-  return game.Get(focusedCell)==Sudoku::empty;
-}
-
-size_t GameArea::GetFocusedCellValue() const
-{
-  assert(game.Get(focusedCell)!=Sudoku::empty);
-
-  return game.Get(focusedCell);
-}
-
-size_t GameArea::GetFocusedCellMarks() const
-{
-  assert(game.Get(focusedCell)==Sudoku::empty);
-
-  return marks[focusedCell];
-}
-
-void GameArea::SetFocusedCellMarks(size_t value)
-{
-  assert(!GetFinishedModel()->Get());
-  assert(IsFocusedCellEditable());
-
-  game.Set(focusedCell,Sudoku::empty);
-  marks[focusedCell]=value;
-
-  focusChangedAction->Trigger();
-  Redraw();
-}
-
-void GameArea::SetFocusedCellValue(size_t value)
-{
-  assert(!GetFinishedModel()->Get());
-  assert(IsFocusedCellEditable());
-
-  game.Set(focusedCell,value);
-  marks[focusedCell]=0;
-
-  SetFinished(game==solution);
-  focusChangedAction->Trigger();
-
-  Redraw();
-
-}
-
-void GameArea::MarkEmpty()
-{
-  assert(!finished->Get());
-
-  for (size_t cell=0; cell<Sudoku::dimension*Sudoku::dimension; cell++) {
-    if (riddle.Get(cell)==Sudoku::empty &&
-        game.Get(cell)==Sudoku::empty &&
-        marks[cell]==0) {
-      marks[cell]=0;
-      for (size_t mark=0; mark<Sudoku::dimension; mark++) {
-        marks[cell]|=(1 << mark);
-      }
-    }
-  }
-
-  Redraw();
-}
-
-void GameArea::Solve()
-{
-  SetFinished(true);
-  Redraw();
-}
-
 void GameArea::Resync(Lum::Base::Model *model, const Lum::Base::ResyncMsg& msg)
 {
   if (model==hintMode) {
     Redraw();
   }
+  else if (model==this->model) {
+    Redraw();
+  }
 
   Object::Resync(model,msg);
 }
