@@ -41,9 +41,11 @@ GenerateDialog::Level::Level(const std::wstring& label,
 
 GenerateDialog::GenerateDialog(size_t points)
  : commitAction(new Lum::Model::Action()),
-   level(new Lum::Model::SizeT(0)),
+   level(new Lum::Model::ULong(0)),
    success(false)
 {
+  GetWindow()->SetScreenOrientationHint(Lum::OS::Window::screenOrientationBothSupported);
+
   levels.push_back(Level(_(L"POINTS_LEVEL_STARTERS",L"For starters"),0));
   levels.push_back(Level(_(L"POINTS_LEVEL_VERY_EASY",L"Very Easy"),1000));
   levels.push_back(Level(_(L"POINTS_LEVEL_EASY",L"Easy"),2500));
